@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWReference.cpp 177 2008-08-23 23:48:45Z vadim $
+  $Id: FWReference.cpp 236 2008-12-25 20:36:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -51,7 +51,7 @@ FWReference::FWReference()
     setPointer(NULL);
 }
 
-FWReference::FWReference(const FWObject *root,bool prepopulate) : FWObject(root,prepopulate)
+FWReference::FWReference(const FWObjectDatabase *root,bool prepopulate) : FWObject(root,prepopulate)
 {
     setPointer(NULL);
 }
@@ -149,7 +149,7 @@ int FWReference::getPointerId()
 }
 
 
-void   FWReference::dump(std::ostream &f,bool recursive,bool brief,int offset)
+void FWReference::dump(std::ostream &f, bool recursive, bool brief, int offset)
 {
     FWObject::dump(f, recursive, brief, offset);
 
@@ -163,3 +163,9 @@ void   FWReference::dump(std::ostream &f,bool recursive,bool brief,int offset)
 	f << string(offset,' ') << "Ptr.id: "   << ptr->getId() <<endl;
     }
 }
+
+FWObject* FWReference::getObject(FWObject* o)
+{
+    if (FWReference::cast(o)!=NULL) return FWReference::cast(o)->getPointer();
+    return o;
+}
