@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IntervalGroup.cpp 202 2008-10-12 04:02:18Z vadim $
+  $Id: IntervalGroup.cpp 236 2008-12-25 20:36:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -36,7 +36,7 @@ using namespace libfwbuilder;
 const char *IntervalGroup::TYPENAME={"IntervalGroup"};
 
 IntervalGroup::IntervalGroup() : Group() {}
-IntervalGroup::IntervalGroup(const FWObject *root,bool prepopulate) :
+IntervalGroup::IntervalGroup(const FWObjectDatabase *root,bool prepopulate) :
     FWObject(root,prepopulate) {}
 
 IntervalGroup::~IntervalGroup() {}
@@ -53,7 +53,7 @@ bool  IntervalGroup::validateChild(FWObject *o)
 FWReference* IntervalGroup::createRef()
 {
 //    FWIntervalReference *ref=new FWIntervalReference();
-    FWIntervalReference *ref=FWIntervalReference::cast(getRoot()->create(FWIntervalReference::TYPENAME));
+    FWIntervalReference *ref = getRoot()->createFWIntervalReference();
     ref->setPointer(this);
     return ref;
 }
