@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectGroup.cpp 202 2008-10-12 04:02:18Z vadim $
+  $Id: ObjectGroup.cpp 236 2008-12-25 20:36:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -39,7 +39,7 @@ using namespace libfwbuilder;
 const char *ObjectGroup::TYPENAME={"ObjectGroup"};
 
 ObjectGroup::ObjectGroup() : Group() {}
-ObjectGroup::ObjectGroup(const FWObject *root,bool prepopulate) :
+ObjectGroup::ObjectGroup(const FWObjectDatabase *root,bool prepopulate) :
     Group(root,prepopulate) {}
 
 ObjectGroup::~ObjectGroup() {}
