@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: RuleElement.cpp 212 2008-10-22 04:45:16Z vadim $
+  $Id: RuleElement.cpp 253 2009-03-26 04:07:36Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -33,6 +33,8 @@
 #include <fwbuilder/Rule.h>
 
 #include <fwbuilder/IPService.h>
+#include <fwbuilder/TagService.h>
+
 #include <fwbuilder/Interval.h>
 #include <fwbuilder/Interface.h>
 #include <fwbuilder/Network.h>
@@ -56,7 +58,7 @@ RuleElement::RuleElement()
     setNeg(false);
 }
 
-RuleElement::RuleElement(const FWObject *root, bool prepopulate) :
+RuleElement::RuleElement(const FWObjectDatabase *root, bool prepopulate) :
     FWObject(root,prepopulate)
 {
     setNeg(false);
@@ -111,7 +113,7 @@ void RuleElement::removeRef(FWObject *obj)
     
     if (getChildrenCount()==0)
     {   // there is nothing left
-	obj=obj->getRoot()->findInIndex( getAnyElementId() );
+	obj = obj->getRoot()->findInIndex( getAnyElementId() );
 	if (obj) addRef(obj);
     }
 }
@@ -121,15 +123,11 @@ bool RuleElement::isAny()
     if (getChildrenCount()!=1) return(false);
 
     FWObject *o;
-    list<FWObject*>::iterator m=begin();
+    list<FWObject*>::iterator m = begin();
     if (  (o=(*m))!=NULL ) 
     {
-	if ( FWObjectReference::isA(o) ||
-	     FWServiceReference::isA(o) ||
-	     FWIntervalReference::isA(o) ) 
-        {
-	    if ((FWReference::cast(o))->getPointerId()==getAnyElementId()) return(true);
-	}
+        if ((FWReference::cast(o))->getPointerId() == getAnyElementId())
+            return(true);
     }
     return(false);
 }
@@ -138,8 +136,7 @@ bool RuleElement::isAny()
 void RuleElement::setAnyElement()
 {
     int any_id = getAnyElementId();
-    
-    FWObject *any=getRoot()->findInIndex( any_id );
+    FWObject *any = getRoot()->findInIndex( any_id );
     if (any)  addRef( any );
 }
 
@@ -157,7 +154,7 @@ void RuleElement::reset()
  *
  * this method is intended for internal use only
  */
-void RuleElement::_initialize(const FWObject *root)
+void RuleElement::_initialize(const FWObjectDatabase *root)
 {
     setRoot(root);
 
@@ -182,7 +179,7 @@ int RuleElement::getAnyElementId()
 
 const char *RuleElementSrc::TYPENAME={"Src"};
 RuleElementSrc::RuleElementSrc() {}
-RuleElementSrc::RuleElementSrc(const FWObject *root,bool prepopulate) :
+RuleElementSrc::RuleElementSrc(const FWObjectDatabase *root,bool prepopulate) :
     ObjectGroup(root,prepopulate),RuleElement(root,prepopulate) 
 {
     if (prepopulate)
@@ -209,7 +206,7 @@ xmlNodePtr RuleElementSrc::toXML(xmlNodePtr parent) throw(FWException)
 
 const char *RuleElementDst::TYPENAME={"Dst"};
 RuleElementDst::RuleElementDst() {}
-RuleElementDst::RuleElementDst(const FWObject *root,bool prepopulate) :
+RuleElementDst::RuleElementDst(const FWObjectDatabase *root,bool prepopulate) :
     ObjectGroup(root,prepopulate),RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -236,7 +233,7 @@ xmlNodePtr RuleElementDst::toXML(xmlNodePtr parent) throw(FWException)
 
 const char *RuleElementSrv::TYPENAME={"Srv"};
 RuleElementSrv::RuleElementSrv() {}
-RuleElementSrv::RuleElementSrv(const FWObject *root,bool prepopulate) :
+RuleElementSrv::RuleElementSrv(const FWObjectDatabase *root,bool prepopulate) :
     ServiceGroup(root,prepopulate),RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -263,7 +260,7 @@ xmlNodePtr RuleElementSrv::toXML(xmlNodePtr parent) throw(FWException)
 
 const char *RuleElementItf::TYPENAME={"Itf"};
 RuleElementItf::RuleElementItf() {}
-RuleElementItf::RuleElementItf(const FWObject *root,bool prepopulate) :
+RuleElementItf::RuleElementItf(const FWObjectDatabase *root,bool prepopulate) :
     RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -298,9 +295,7 @@ bool RuleElementItf::validateChild(FWObject *o)
       bool all_intf = true;
       for (FWObject::iterator i=o->begin(); i!=o->end(); ++i)
       {
-          FWObject *o1= *i;
-          if (FWReference::cast(o1)!=NULL)
-              o1=FWReference::cast(o1)->getPointer();
+          FWObject *o1 = FWReference::getObject(*i);
           if (!Interface::isA(o1)) { all_intf = false; break; }
       }
       return all_intf;  // group is allowed if all members are interfaces
@@ -350,7 +345,7 @@ bool RuleElementItf::checkItfChildOfThisFw(FWObject *o)
 
 const char *RuleElementOSrc::TYPENAME={"OSrc"};
 RuleElementOSrc::RuleElementOSrc() {}
-RuleElementOSrc::RuleElementOSrc(const FWObject *root,bool prepopulate) :
+RuleElementOSrc::RuleElementOSrc(const FWObjectDatabase *root,bool prepopulate) :
     RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -377,7 +372,7 @@ xmlNodePtr RuleElementOSrc::toXML(xmlNodePtr parent) throw(FWException)
 
 const char *RuleElementODst::TYPENAME={"ODst"};
 RuleElementODst::RuleElementODst() {}
-RuleElementODst::RuleElementODst(const FWObject *root,bool prepopulate) :
+RuleElementODst::RuleElementODst(const FWObjectDatabase *root,bool prepopulate) :
     RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -404,7 +399,7 @@ xmlNodePtr RuleElementODst::toXML(xmlNodePtr parent) throw(FWException)
 
 const char *RuleElementOSrv::TYPENAME={"OSrv"};
 RuleElementOSrv::RuleElementOSrv() {}
-RuleElementOSrv::RuleElementOSrv(const FWObject *root,bool prepopulate) :
+RuleElementOSrv::RuleElementOSrv(const FWObjectDatabase *root,bool prepopulate) :
     RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -433,7 +428,7 @@ xmlNodePtr RuleElementOSrv::toXML(xmlNodePtr parent) throw(FWException)
 
 const char *RuleElementTSrc::TYPENAME={"TSrc"};
 RuleElementTSrc::RuleElementTSrc() {}
-RuleElementTSrc::RuleElementTSrc(const FWObject *root,bool prepopulate) :
+RuleElementTSrc::RuleElementTSrc(const FWObjectDatabase *root,bool prepopulate) :
     RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -460,7 +455,7 @@ xmlNodePtr RuleElementTSrc::toXML(xmlNodePtr parent) throw(FWException)
 
 const char *RuleElementTDst::TYPENAME={"TDst"};
 RuleElementTDst::RuleElementTDst() {}
-RuleElementTDst::RuleElementTDst(const FWObject *root,bool prepopulate) :
+RuleElementTDst::RuleElementTDst(const FWObjectDatabase *root,bool prepopulate) :
     RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -487,7 +482,7 @@ xmlNodePtr RuleElementTDst::toXML(xmlNodePtr parent) throw(FWException)
 
 const char *RuleElementTSrv::TYPENAME={"TSrv"};
 RuleElementTSrv::RuleElementTSrv()  {}
-RuleElementTSrv::RuleElementTSrv(const FWObject *root,bool prepopulate) :
+RuleElementTSrv::RuleElementTSrv(const FWObjectDatabase *root,bool prepopulate) :
     RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -511,12 +506,31 @@ xmlNodePtr RuleElementTSrv::toXML(xmlNodePtr parent) throw(FWException)
     return me;
 }
 
+bool RuleElementTSrv::validateChild(FWObject *o)
+{
+  if (FWObjectReference::cast(o)!=NULL) return true;
+  if ( o->getId() == getAnyElementId()) return true;
+
+  // TagService is not allowed in translated service
+  if (TagService::isA(o)) return false;
+
+  if (ServiceGroup::cast(o)!=NULL)
+  {
+      for (FWObject::iterator i=o->begin(); i!=o->end(); ++i)
+      {
+          FWObject *o1 = FWReference::getObject(*i);
+          if (!validateChild(o1)) return false;
+      }
+  }
+  return true;
+}
+
 
 
 
 const char *RuleElementInterval::TYPENAME={"When"};
 RuleElementInterval::RuleElementInterval() {}
-RuleElementInterval::RuleElementInterval(const FWObject *root,bool prepopulate) :
+RuleElementInterval::RuleElementInterval(const FWObjectDatabase *root,bool prepopulate) :
     RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -545,7 +559,7 @@ xmlNodePtr RuleElementInterval::toXML(xmlNodePtr parent) throw(FWException)
 
 const char *RuleElementRDst::TYPENAME={"RDst"};
 RuleElementRDst::RuleElementRDst() {}
-RuleElementRDst::RuleElementRDst(const FWObject *root,bool prepopulate) :
+RuleElementRDst::RuleElementRDst(const FWObjectDatabase *root,bool prepopulate) :
     RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -571,7 +585,7 @@ xmlNodePtr RuleElementRDst::toXML(xmlNodePtr parent) throw(FWException)
 
 const char *RuleElementRGtw::TYPENAME={"RGtw"};
 RuleElementRGtw::RuleElementRGtw() {}
-RuleElementRGtw::RuleElementRGtw(const FWObject *root,bool prepopulate) :
+RuleElementRGtw::RuleElementRGtw(const FWObjectDatabase *root,bool prepopulate) :
     RuleElement(root,prepopulate)
 {
     if (prepopulate)
@@ -639,7 +653,7 @@ bool RuleElementRGtw::checkSingleIPAdress(FWObject *o) {
 
 const char *RuleElementRItf::TYPENAME={"RItf"};
 RuleElementRItf::RuleElementRItf()  {}
-RuleElementRItf::RuleElementRItf(const FWObject *root,bool prepopulate) :
+RuleElementRItf::RuleElementRItf(const FWObjectDatabase *root,bool prepopulate) :
     RuleElementItf(root,prepopulate) {}
 
 /**
