@@ -4,9 +4,9 @@
 
                  Copyright (C) 2002 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Compiler.cpp 216 2008-10-25 17:19:39Z vadim $
+  $Id: Compiler.cpp 237 2008-12-28 07:12:17Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -44,7 +44,6 @@
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/IPv6.h"
-#include "fwbuilder/InterfacePolicy.h"
 #include "fwbuilder/DNSName.h"
 #include "fwbuilder/MultiAddress.h"
 
@@ -100,8 +99,8 @@ int Compiler::prolog()
     FWObjectTypedChildIterator j=fw->findByType(Interface::TYPENAME);
     for ( ; j!=j.end(); ++j )
     {
-        Interface *interface_=Interface::cast(*j);
-        fw_interfaces[interface_->getId()]=interface_;
+        Interface *interface_ = Interface::cast(*j);
+        fw_interfaces[interface_->getId()] = interface_;
     }
     
     fw_id=fw->getId();
@@ -281,33 +280,6 @@ void Compiler::compile()
 
 }
 
-bool Compiler::MatchesAddressFamily(FWObject *o)
-{
-    if (Address::cast(o))
-    {
-        const  InetAddr *inet_addr = Address::cast(o)->getAddressPtr();
-        if (inet_addr)
-        {
-            if (ipv6)
-            {
-                if (inet_addr->isV6()) return true;
-            } else
-            {
-                if (inet_addr->isV4()) return true;
-            }
-        } else
-        {
-            // Address object with no ip address (e.g. dynamic interface
-            // or run-time address table)
-            return true;
-        }
-        return false;
-    }
-    // not an address object at all
-    return true;
-}
-
-
 void Compiler::_expand_group_recursive(FWObject *o, list<FWObject*> &ol)
 {
 /* special case: MultiAddress. This class inherits ObjectGroup, but
@@ -322,9 +294,7 @@ void Compiler::_expand_group_recursive(FWObject *o, list<FWObject*> &ol)
     {
 	for (FWObject::iterator i2=o->begin(); i2!=o->end(); ++i2)
         {
-	    FWObject *o1 = *i2;
-	    if (FWReference::cast(o1)!=NULL)
-                o1=FWReference::cast(o1)->getPointer();
+            FWObject *o1 = FWReference::getObject(*i2);
 	    assert(o1);
 
 	    _expand_group_recursive(o1, ol);
@@ -337,7 +307,8 @@ void Compiler::_expand_group_recursive(FWObject *o, list<FWObject*> &ol)
             ol.push_back( o );
         } else
         {
-            if (Address::cast(o) && Address::cast(o)->hasInetAddress())
+            Address *oaddr = Address::cast(o);
+            if (oaddr && oaddr->hasInetAddress())
             {
                 if (MatchesAddressFamily(o))
                 {
@@ -363,8 +334,7 @@ void Compiler::expandGroupsInRuleElement(RuleElement *s)
     list<FWObject*> cl;
     for (FWObject::iterator i1=s->begin(); i1!=s->end(); ++i1)
     {
-        FWObject *o= *i1;
-        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i1);
         assert(o);
         _expand_group_recursive(o, cl);
     }
@@ -388,22 +358,20 @@ void Compiler::_expand_addr_recursive(Rule *rule, FWObject *s,
 
     for (FWObject::iterator i1=s->begin(); i1!=s->end(); ++i1) 
     {
-	FWObject *o= *i1;
-	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i1);
 	assert(o);
 
+        Address *addr = Address::cast(o);
+
         // this condition includes Host, Firewall and Interface
-        if (Address::cast(o) &&
-            !Address::cast(o)->hasInetAddress())
+        if (addr && !addr->hasInetAddress())
         {
             addrlist.push_back(o);
             continue;
         }
 
         // IPv4, IPv6, Network, NetworkIPv6
-        if (Address::cast(o) &&
-            Address::cast(o)->hasInetAddress() &&
-            MatchesAddressFamily(o))
+        if (addr && addr->hasInetAddress() && MatchesAddressFamily(o))
         {            
             addrlist.push_back(o);
             continue;
@@ -428,9 +396,9 @@ void Compiler::_expand_addr_recursive(Rule *rule, FWObject *s,
         for (list<FWObject*>::iterator i2=addrlist.begin();
              i2!=addrlist.end(); ++i2)
         {
-            if (Interface::cast(*i2)!=NULL)
+            Interface *i2itf = Interface::cast(*i2);
+            if (i2itf)
             {
-                Interface *interface_ = Interface::cast(*i2);
 /*
  * Special case is loopback interface - skip it, but only if this rule is
  * not attached to loopback!
@@ -440,13 +408,13 @@ void Compiler::_expand_addr_recursive(Rule *rule, FWObject *s,
  * firewall object, then skip it unless the rule is attached to
  * loopback interface.
  */
-                if (interface_->isLoopback())
+                if (i2itf->isLoopback())
                 {
                     if (RuleElement::cast(s) || on_loopback)
-                        _expandInterface(interface_,ol);
+                        _expandInterface(i2itf, ol);
                 } else
 // this is not a loopback interface                
-                    _expandInterface(interface_,ol);
+                    _expandInterface(i2itf, ol);
 
                 continue;
             }
@@ -455,7 +423,7 @@ void Compiler::_expand_addr_recursive(Rule *rule, FWObject *s,
     }
 }
 
-void Compiler::_expandInterface(Interface *iface,  std::list<FWObject*> &ol)
+void Compiler::_expandInterface(Interface *iface, std::list<FWObject*> &ol)
 {
 /*
  * if this is unnumbered interface or a bridge port, then do not use it
@@ -479,12 +447,11 @@ void Compiler::_expandInterface(Interface *iface,  std::list<FWObject*> &ol)
  */
     FWObject  *p;
     FWOptions *hopt;
-    p=iface->getParent();
-    bool use_mac= (Host::cast(p)!=NULL && 
-                   (hopt=Host::cast(p)->getOptionsObject())!=NULL &&
+    p = iface->getParent();
+    Host *hp = Host::cast(p);
+    bool use_mac= (hp && (hopt = hp->getOptionsObject())!=NULL &&
                    hopt->getBool("use_mac_addr_filter") ); 
 
-
     for (FWObject::iterator i1=iface->begin(); i1!=iface->end(); ++i1) 
     {
 	FWObject *o= *i1;
@@ -541,28 +508,27 @@ void Compiler::_expandAddressRanges(Rule*, FWObject *re)
     list<FWObject*> cl;
     for (FWObject::iterator i1=re->begin(); i1!=re->end(); ++i1) 
     {
-	FWObject *o= *i1;
-	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i1);
 	assert(o!=NULL);
 
         // if this is address range, check if it matches current address
         // family. If it is not address range, put it back into the rule element
         // If it is address range but it does not match address family,
         // throw it away.
-	if (AddressRange::cast(o)!=NULL)
+        AddressRange *aro = AddressRange::cast(o);
+	if (aro)
         {
             if (MatchesAddressFamily(o))
             {
-                InetAddr a1 = AddressRange::cast(o)->getRangeStart();
-                InetAddr a2 = AddressRange::cast(o)->getRangeEnd();
+                InetAddr a1 = aro->getRangeStart();
+                InetAddr a2 = aro->getRangeEnd();
                 vector<InetAddrMask> vn = 
                     libfwbuilder::convertAddressRange(a1,a2);
 
                 for (vector<InetAddrMask>::iterator i=vn.begin();
                      i!=vn.end(); i++)
                 {
-                    Network *h;
-                    h= Network::cast(dbcopy->create(Network::TYPENAME) );
+                    Network *h = dbcopy->createNetwork();
                     h->setName(string("%n-")+(*i).toString()+string("%") );
                     h->setNetmask(*(i->getNetmaskPtr()));
                     h->setAddress(*(i->getAddressPtr()));
@@ -590,267 +556,15 @@ void Compiler::normalizePortRange(int &rs,int &re)
 }
 
 
-bool Compiler::_complexMatchWithAddress(const InetAddr *obj1_addr,
-                                        Interface *iface,
-                                        const string &address_type,
-                                        bool recognize_broadcasts)
-{
-    FWObjectTypedChildIterator k = iface->findByType(address_type);
-    for ( ; k!=k.end(); ++k ) 
-    {
-        Address *addr_obj = Address::cast(*k);
-                    
-        if ( *(addr_obj->getAddressPtr())==*(obj1_addr) ) return true;
-        const InetAddr *addr = addr_obj->getAddressPtr();
-        const InetAddr *netm = addr_obj->getNetmaskPtr();
-        if (addr)
-        {
-            InetAddrMask n(*addr, *netm);
-/*
- * bug #1040773: need to match network address as well as
- * broadcast. Packets sent to the network address (192.168.1.0 for net
- * 192.168.1.0/24) go in the broadcast frame and behave just like IP
- * broadcast packets (sent to 192.168.1.1255 for the same net)
- *
- */
-            if (recognize_broadcasts && (
-                    *(n.getNetworkAddressPtr())==*(obj1_addr) ||
-                    *(n.getBroadcastAddressPtr())==*(obj1_addr)
-                )) return true;
-        }
-    }
-    return false;
-}
-
-bool Compiler::_complexMatchWithInterface(Address   *obj1,
-                                          Interface *iface,
-                                          bool recognize_broadcasts)
-{
-    const InetAddr *obj1_addr = obj1->getAddressPtr();
-
-    if ( physAddress::isA(obj1))
-    {
-        physAddress *obj1_pa =physAddress::cast(obj1);
-        physAddress *iface_pa=iface->getPhysicalAddress();
-        if (iface_pa!=NULL && 
-            obj1_pa->getPhysAddress()==iface_pa->getPhysAddress()) return true;
-    }
-
-    if ( iface->isRegular() )
-    {
-        string address_type = (ipv6) ? IPv6::TYPENAME : IPv4::TYPENAME;
-        return _complexMatchWithAddress(
-            obj1_addr, iface, address_type, recognize_broadcasts);
-    }
-    return false;
-}
-
-/**
- * returns true if :
- * 1. obj1 is the same as obj2, or 
- * 2. any child of obj2 or 
- * 3. its address matches that of any obj2's interfaces, or 
- * 4. its address matches broadcast address of one of the interfaces
- * 5. address of obj1 is a broadcast (255.255.255.255)
- * 6. address of obj1 is a multicast ( added 09/15/02, --vk )
- */
-bool Compiler::complexMatch(Address *obj1,
-                            Address *obj2,
-                            bool recognize_broadcasts,
-                            bool recognize_multicasts)
-{
-    if (Network::isA(obj1) || NetworkIPv6::isA(obj1))
-    {
-        /*
-         * bug #1055937: "Any->all_multicasts not in INPUT Chain"
-         * Need to check for multicast networks. We assume they always
-         * match if obj2 is firewall
-         */
-        //Network *nobj1 = Network::cast(obj1);
-        const InetAddr *inet_addr = obj1->getAddressPtr();
-        if (inet_addr)
-        {
-            if (inet_addr->isMulticast() && Firewall::isA(obj2))
-                return true;
-            /*
-             * need to check for network object with mask 255.255.255.255
-             * Such objects are created by the method that expands address
-             * ranges, and some often used ranges trigger that (like
-             * "255.255.255.255-255.255.255.255" or "0.0.0.0-0.0.0.0")
-             */
-            if (!obj1->getNetmaskPtr()->isHostMask())
-                return false;
-        } else
-            warning(string("Network object with no InetAddr: id=") +
-                    FWObjectDatabase::getStringId(obj1->getId()) +
-                    "  type=" + obj1->getTypeName() +
-                    "  name=" + obj1->getName());
-    }
-/*
- * if we are matching two firewalls, the positive match is only if
- * their IDs are the same (so it is the same firewall)
- */
-    if (Firewall::isA(obj1) && Firewall::isA(obj2))
-        return (obj1->getId()==obj2->getId());
-/*
- *  match only if object 'obj1' is Network, AddressRange or Host/Firewall
- *  with one interface. Simple and general rule is object 'obj1' should
- *  have zero or one interface child object.
- */
-    list<FWObject*> l;
-    l=obj1->getByType(Interface::TYPENAME);
-    if (l.size()>1) return false;
-
-/*
- *  if obj1 is Interface, match only if it a child of obj2, otherwise
- *  match if it has single address
- */
-//    if (obj1->getId()==obj2->getId() ||
-//        obj2->getById( obj1->getId() , true )!=NULL ) return true;
-
-
-    if (obj1->getId()==obj2->getId()) return true;
-
-    FWObject *p=obj1;
-    while ( (p=p->getParent())!=NULL )  // it is faster to search this way
-        if (p->getId()==obj2->getId()) return true;
-
-    // TODO: this can be expensive since we create and return temporary
-    // list object only to get its size. Need method FWObject::countByType
-    if ((obj1->getByType(IPv4::TYPENAME)).size()>1) return false;
-    if ((obj1->getByType(IPv6::TYPENAME)).size()>1) return false;
-
-    const InetAddr *obj1_addr = obj1->getAddressPtr();
-    // obj1_addr may be NULL if obj1 does not have any real address,
-    // one case when this happens is when obj1 is physAddress
-    if (obj1_addr)
-    {
-        if (!obj1_addr->isAny() && 
-            ( (recognize_broadcasts && obj1_addr->isBroadcast()) || 
-              (recognize_multicasts && obj1_addr->isMulticast()) )
-        ) return true;
-
-        if (Interface::cast(obj1)!=NULL && !Interface::cast(obj1)->isRegular())
-            return false;
-
-        if (Interface::cast(obj2)!=NULL)
-        {
-            return _complexMatchWithInterface(obj1,Interface::cast(obj2));
-        }else
-        {
-            FWObjectTypedChildIterator j=obj2->findByType(Interface::TYPENAME);
-            for ( ; j!=j.end(); ++j ) 
-            {
-                Interface *iface=Interface::cast(*j);
-                if (_complexMatchWithInterface(obj1,iface)) return true;
-            }
-        }
-    }
-
-    return false;
-}
-
-/**
- * This method finds interface of obj2 (which is usually
- * firewall object, but not necessarily so) which is connected
- * to the subnet on which obj1 is located.
- */
-Interface* Compiler::findInterfaceFor(const Address *obj1, const Address *obj2)
-{
-    FWObjectTypedChildIterator j=obj2->findByType(Interface::TYPENAME);
-    for ( ; j!=j.end(); ++j ) 
-    {
-	Interface *iface=Interface::cast(*j);
-	assert(iface);
-
-        if (iface->getId()      == obj1->getId() )      return iface;
-
-        if ( iface->isRegular() && obj1->getAddressPtr() != NULL)
-        {
-            if (obj1->getAddressPtr()->isV4())
-            {
-                FWObjectTypedChildIterator k= iface->findByType(IPv4::TYPENAME);
-                for ( ; k!=k.end(); ++k ) 
-                {
-                    Address *addr = Address::cast(*k);
-                    assert(addr);
-                    if (checkIfAddressesMatch(addr, obj1)) return iface;
-                }
-            }
-            if (obj1->getAddressPtr()->isV6())
-            {
-                FWObjectTypedChildIterator k= iface->findByType(IPv6::TYPENAME);
-                for ( ; k!=k.end(); ++k ) 
-                {
-                    Address *addr = Address::cast(*k);
-                    assert(addr);
-                    if (checkIfAddressesMatch(addr, obj1)) return iface;
-                }
-            }
-        }
-    }
-    return NULL;
-}
-
-FWObject* Compiler::findAddressFor(const Address *obj1,const Address *obj2)
-{
-    FWObjectTypedChildIterator j=obj2->findByType(Interface::TYPENAME);
-    for ( ; j!=j.end(); ++j ) 
-    {
-	Interface *iface=Interface::cast(*j);
-	assert(iface);
-
-        if (iface->getId() == obj1->getId() )      return iface;
-
-        if ( iface->isRegular() )
-        {
-            if (obj1->getAddressPtr()->isV4())
-            {
-                FWObjectTypedChildIterator k= iface->findByType(IPv4::TYPENAME);
-                for ( ; k!=k.end(); ++k ) 
-                {
-                    Address *addr = Address::cast(*k);
-                    assert(addr);
-                    if (checkIfAddressesMatch(addr, obj1)) return (*k);
-                }
-            }
-            if (obj1->getAddressPtr()->isV6())
-            {
-                FWObjectTypedChildIterator k= iface->findByType(IPv6::TYPENAME);
-                for ( ; k!=k.end(); ++k ) 
-                {
-                    Address *addr = Address::cast(*k);
-                    assert(addr);
-                    if (checkIfAddressesMatch(addr, obj1)) return (*k);
-                }
-            }
-        }
-    }
-    return NULL;
-}
-        
-bool Compiler::checkIfAddressesMatch(const Address *a1, const Address *a2)
-{
-    if (a1->getId() == a2->getId()) return true;
-    if (*(a1->getAddressPtr()) == *(a2->getAddressPtr()) ) return true;
-    if ((Network::constcast(a2)!=NULL ||
-         NetworkIPv6::constcast(a2)!=NULL ||
-         Interface::constcast(a2->getParent())) &&
-        a2->belongs(*(a1->getAddressPtr()))) return true;
-    if ((Network::constcast(a1)!=NULL ||
-         NetworkIPv6::constcast(a1)!=NULL ||
-         Interface::constcast(a1->getParent())) &&
-        a1->belongs(*(a2->getAddressPtr()))) return true;
-    return false;
-}
-
 
 void Compiler::debugRule()
 {
-    for(FWObject::iterator i=combined_ruleset->begin(); i!=combined_ruleset->end(); i++) {
+    for (FWObject::iterator i=combined_ruleset->begin();
+         i!=combined_ruleset->end(); i++)
+    {
 	Rule *rule = Rule::cast( *i );
-
-        if ( rule->getPosition()==debug_rule ) {
+        if ( rule->getPosition()==debug_rule )
+        {
             cout << debugPrintRule(rule);
             cout << endl;
         }
@@ -920,11 +634,9 @@ bool Compiler::Begin::processNext()
              i!=compiler->combined_ruleset->end(); ++i)
         {
             Rule *rule=Rule::cast(*i);
-
-            Rule  *r= Rule::cast(compiler->dbcopy->create(rule->getTypeName()) );
+            Rule  *r= Rule::cast(compiler->dbcopy->create(rule->getTypeName()));
             compiler->temp_ruleset->add(r);
             r->duplicate(rule);
-
             tmp_queue.push_back( r );
         }
         init=true;
@@ -1033,23 +745,20 @@ bool Compiler::splitIfRuleElementMatchesFW::processNext()
 
     for (list<FWObject*>::iterator i1=re->begin(); nre>1 && i1!=re->end(); ++i1) {
 
-	FWObject *o   = *i1;
-	FWObject *obj = NULL;
-	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
-        Address *a=Address::cast(obj);
+        FWObject *obj = FWReference::getObject(*i1);
+        Address *a = Address::cast(obj);
         assert(a!=NULL);
 
-//        InetAddr obj_addr=a->getAddress();
-
-        if (compiler->complexMatch(a,compiler->fw))
+        if (compiler->complexMatch(a, compiler->fw))
         {
-	    cl.push_back(o);   // can not remove right now because remove invalidates iterator
+	    cl.push_back(*i1);
             nre--;
-
-	    Rule  *new_rule= Rule::cast(compiler->dbcopy->create(rule->getTypeName()) );
+	    Rule  *new_rule = Rule::cast(
+                compiler->dbcopy->create(rule->getTypeName()) );
 	    compiler->temp_ruleset->add(new_rule);
 	    new_rule->duplicate(rule);
-            RuleElement *new_re = RuleElement::cast(new_rule->getFirstByType(re_type));
+            RuleElement *new_re = RuleElement::cast(
+                new_rule->getFirstByType(re_type));
 	    new_re->clearChildren();
 	    new_re->setAnyElement();
 	    new_re->addRef( a );
@@ -1083,11 +792,9 @@ bool Compiler::eliminateDuplicatesInRE::processNext()
 
     vector<FWObject*> cl;
 
-    for(list<FWObject*>::iterator i=re->begin(); i!=re->end(); ++i) {
-        FWObject *o= *i;
-	FWObject *obj = NULL;
-        if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
-
+    for(list<FWObject*>::iterator i=re->begin(); i!=re->end(); ++i)
+    {
+        FWObject *obj = FWReference::getObject(*i);
         comparator->set(obj);
 
         bool found=false;
@@ -1096,11 +803,6 @@ bool Compiler::eliminateDuplicatesInRE::processNext()
             if ( (*comparator)( (*i1) ) ) { found=true; break; }
         }
         if (!found) cl.push_back(obj);
-
-//        if (std::find_if(cl.begin(),cl.end(),*comparator)==cl.end())
-//        {
-//            cl.push_back(obj);
-//        }
     }
     if (!cl.empty())
     {
@@ -1118,8 +820,7 @@ void  Compiler::recursiveGroupsInRE::isRecursiveGroup(int grid, FWObject *obj)
 {
     for (FWObject::iterator i=obj->begin(); i!=obj->end(); i++) 
     {
-	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i);
         if (Group::cast(o)!=NULL)
         {
             if (o->getId()==grid)
@@ -1146,8 +847,7 @@ bool Compiler::recursiveGroupsInRE::processNext()
     std::list<FWObject*> cl;
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
-	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i);
         if (Group::cast(o)!=NULL)  isRecursiveGroup(o->getId(),o);
     }
 
@@ -1166,8 +866,8 @@ int  Compiler::emptyGroupsInRE::countChildren(FWObject *obj)
     int n=0;
     for (FWObject::iterator i=obj->begin(); i!=obj->end(); i++) 
     {
-	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i);
+
         // Check if this is a group, if yes, then count its children
         // recursively. Group itself does not count since it can be
         // empty, too.  However if this is MultiAddress object with
@@ -1201,12 +901,10 @@ bool Compiler::emptyGroupsInRE::processNext()
     std::list<FWObject*> cl;
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
-	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i);
         if ( MultiAddress::cast(o)!=NULL && MultiAddress::cast(o)->isRunTime())
             continue;
 
-        //if (Group::cast(o)!=NULL && MultiAddress::cast(o)==NULL && countChildren(o)==0)
         if (Group::cast(o)!=NULL && countChildren(o)==0)
             cl.push_back(o);
     }
@@ -1284,8 +982,7 @@ bool Compiler::swapMultiAddressObjectsInRE::processNext()
     list<MultiAddress*> cl;
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
     {
-        FWObject *o= *i;
-        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i);
         if (MultiAddress::cast(o)!=NULL && MultiAddress::cast(o)->isRunTime())
             cl.push_back(MultiAddress::cast(o));
     }
@@ -1347,8 +1044,7 @@ bool Compiler::FindAddressFamilyInRE(FWObject *parent, bool ipv6)
 
     for (FWObject::iterator i=parent->begin(); i!=parent->end(); i++) 
     {
-        FWObject *o= *i;
-        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i);
         if (FindAddressFamilyInRE(o, ipv6)) return true;
     }
     return false;
@@ -1360,26 +1056,19 @@ void Compiler::DropAddressFamilyInRE(RuleElement *rel, bool drop_ipv6)
     list<FWObject*> objects_to_remove;
     for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
     {
-        FWObject *o= *i;
-        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i);
         // skip "Any"
-        if (o->getId() == FWObjectDatabase::ANY_ADDRESS_ID)
-            continue;
+        if (o->getId() == FWObjectDatabase::ANY_ADDRESS_ID) continue;
 
-        if (drop_ipv6)
+        if (Address::cast(o) && Address::cast(o)->hasInetAddress())
         {
-            if (Address::cast(o) && Address::cast(o)->hasInetAddress())
+            const  InetAddr *inet_addr = Address::cast(o)->getAddressPtr();
+            if (inet_addr)
             {
-                const  InetAddr *inet_addr = Address::cast(o)->getAddressPtr();
-                if (inet_addr && inet_addr->isV6())
+                if (drop_ipv6 && inet_addr->isV6())
                     objects_to_remove.push_back(o);
-            }
-        } else
-        {
-            if (Address::cast(o) && Address::cast(o)->hasInetAddress())
-            {
-                const  InetAddr *inet_addr = Address::cast(o)->getAddressPtr();
-                if (inet_addr && inet_addr->isV4())
+
+                if (!drop_ipv6 && inet_addr->isV4())
                     objects_to_remove.push_back(o);
             }
         }
@@ -1395,20 +1084,21 @@ void Compiler::DropByServiceTypeInRE(RuleElement *rel, bool drop_ipv6)
     list<FWObject*> objects_to_remove;
     for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
     {
-        FWObject *o= *i;
-        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i);
         // skip "Any"
-        if (o->getId() == FWObjectDatabase::ANY_SERVICE_ID)
-            continue;
+        if (o->getId() == FWObjectDatabase::ANY_SERVICE_ID) continue;
 
-        // Note that all service objects except for ICMPService can be
-        // used in both ipv4 and ipv6 contexts.
+        Service *svc = Service::cast(o);
+        assert(svc);
+
+        // Note that all service objects except for ICMPService and
+        // CustomService can be used in both ipv4 and ipv6 contexts.
         if (drop_ipv6)
         {
-            if (ICMP6Service::isA(o)) objects_to_remove.push_back(o);
+            if (svc->isV6Only()) objects_to_remove.push_back(o);
         } else
         {
-            if (ICMPService::isA(o)) objects_to_remove.push_back(o);
+            if (svc->isV4Only()) objects_to_remove.push_back(o);
         }
     }
 
@@ -1417,15 +1107,13 @@ void Compiler::DropByServiceTypeInRE(RuleElement *rel, bool drop_ipv6)
         rel->removeRef(*i);
 }
 
-
 bool Compiler::catchUnnumberedIfaceInRE(RuleElement *re)
 {
-    bool err=false;
+    bool err = false;
     Interface *iface;
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
-	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        FWObject *o = FWReference::getObject(*i);
         if (o==NULL)
         {
             Rule *rule = Rule::cast(re->getParent());
@@ -1444,129 +1132,82 @@ bool Compiler::catchUnnumberedIfaceInRE(RuleElement *re)
     return err;
 }
 
-
 Address* Compiler::getFirstSrc(PolicyRule *rule)
 {
-    RuleElementSrc *src=rule->getSrc();
-
-    FWObject *o=src->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    RuleElementSrc *src = rule->getSrc();
+    FWObject *o = FWReference::getObject(src->front());
     return Address::cast(o);
 }
 
 Address* Compiler::getFirstDst(PolicyRule *rule)
 {
-    RuleElementDst *dst=rule->getDst();
-
-    FWObject *o=dst->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    RuleElementDst *dst = rule->getDst();
+    FWObject *o = FWReference::getObject(dst->front());
     return Address::cast(o);
 }
 
 Service* Compiler::getFirstSrv(PolicyRule *rule)
 {
-    RuleElementSrv *srv=rule->getSrv();
-
-    FWObject *o=srv->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    RuleElementSrv *srv = rule->getSrv();
+    FWObject *o = FWReference::getObject(srv->front());
     return Service::cast(o);
 }
 
 Interval* Compiler::getFirstWhen(PolicyRule *rule)
 {
-    RuleElementInterval *when=rule->getWhen();
+    RuleElementInterval *when = rule->getWhen();
     if (when==NULL) return NULL;  // when is optional element
-
-    FWObject *o=when->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    FWObject *o = FWReference::getObject(when->front());
     return Interval::cast(o);
 }
 
 Interface* Compiler::getFirstItf(PolicyRule *rule)
 {
-    RuleElementItf *itf=rule->getItf();
+    RuleElementItf *itf = rule->getItf();
     if (itf==NULL) return NULL;  // itf is optional element
-
-    FWObject *o=itf->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    FWObject *o = FWReference::getObject(itf->front());
     return Interface::cast(o);
 }
 
 Address* Compiler::getFirstOSrc(NATRule *rule)
 {
-    RuleElementOSrc *osrc=rule->getOSrc();
-
-    FWObject *o=osrc->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    RuleElementOSrc *osrc = rule->getOSrc();
+    FWObject *o = FWReference::getObject(osrc->front());
     return Address::cast(o);
 }
 
 Address* Compiler::getFirstODst(NATRule *rule)
 {
-    RuleElementODst *odst=rule->getODst();
-
-    FWObject *o=odst->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    RuleElementODst *odst = rule->getODst();
+    FWObject *o = FWReference::getObject(odst->front());
     return Address::cast(o);
 }
 
 Service* Compiler::getFirstOSrv(NATRule *rule)
 {
-    RuleElementOSrv *osrv=rule->getOSrv();
-
-    FWObject *o=osrv->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    RuleElementOSrv *osrv = rule->getOSrv();
+    FWObject *o = FWReference::getObject(osrv->front());
     return Service::cast(o);
 }
 
-
-
 Address* Compiler::getFirstTSrc(NATRule *rule)
 {
-    RuleElementTSrc *tsrc=rule->getTSrc();
-
-    FWObject *o=tsrc->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    RuleElementTSrc *tsrc = rule->getTSrc();
+    FWObject *o = FWReference::getObject(tsrc->front());
     return Address::cast(o);
 }
 
 Address* Compiler::getFirstTDst(NATRule *rule)
 {
-    RuleElementTDst *tdst=rule->getTDst();
-
-    FWObject *o=tdst->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    RuleElementTDst *tdst = rule->getTDst();
+    FWObject *o = FWReference::getObject(tdst->front());
     return Address::cast(o);
 }
 
 Service* Compiler::getFirstTSrv(NATRule *rule)
 {
-    RuleElementTSrv *tsrv=rule->getTSrv();
-
-    FWObject *o=tsrv->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=FWReference::cast(o)->getPointer();
-
+    RuleElementTSrv *tsrv = rule->getTSrv();
+    FWObject *o = FWReference::getObject(tsrv->front());
     return Service::cast(o);
 }
 
